package examples.managed;

import examples.managed.ejb.*;
import javax.rmi.PortableRemoteObject;
import javax.naming.*;
import java.util.Properties;


/**
 * A simple client application to lookup and invoke the InventoryBean EJB used
 * to demonstrate the functionality of the Resource Adapter for CrossWorlds.
 * To use this client, the EJB must be deployed and running in your appserver.
 */
public class Client {


    /**
     * Performs a lookup of the example EJB and executes its
     * <code>retrievePrice</code> method.
     */
    static public void main( String[] args ) {
        if( args.length != 4 ) {
            System.out.println( "Usage: <provider URL> <initial context factory class> <EJB home JNDI name> <itemId>" );
            System.exit(1);
        }

        String providerURL = args[0];
        String icf = args[1];
        String jndiName = args[2];
        String itemId = args[3];

        InitialContext ic = getInitialContextFactory( providerURL, icf );
        InventoryHome home = lookupInventoryHome( ic, jndiName );

        Inventory bean = null;
        try {
            System.out.println( "Creating bean..." );
            bean = home.create();
            System.out.println( "Created bean. Retrieving price..." );
            double price = bean.retrievePrice( itemId );
            System.out.println( "Price of " + itemId + " = " + price );
            bean.remove();
        }
        catch( Exception e ) {
            System.err.println( "An error occurred while attempting to retrieve price of " + itemId +
                                ". Error:\n" + e + "\nCheck application server logs for possibly more details." +
                                "\nVerify that CrossWorlds ICS is running." );

            //last ditch effort to clean-up before exiting
            try { bean.remove(); } catch( Exception x ) { }
            System.exit(1);
        }

        System.out.println( "Done" );
    }


    /**
     * Returns an intial context in which to lookup the EJB.
     */
    static private InitialContext getInitialContextFactory( String providerURL, String icf )
    {
        System.out.println( "Establishing initial context to lookup EJB..." );
        InitialContext ctx = null;
        try {
            Properties p = new Properties();
            p.put( Context.INITIAL_CONTEXT_FACTORY, icf );
            p.put( Context.PROVIDER_URL, providerURL );
            ctx = new InitialContext( p );
            System.out.println( "Established initial context." );
        }
        catch( NamingException ne ) {
            System.err.println( "Unable to establish initial context to lookup bean: " + ne +
                                ". Verify that your appplication server is running." );
            System.exit(1);
        }
        return ctx;
    }


    /**
     * Retrieves the EJB home stub from the context <code>ctx</code>.
     */
    static private InventoryHome lookupInventoryHome( Context ctx, String name )
    {
        System.out.println( "Looking up object \"" + name + "\"..." );
        InventoryHome home = null;
        try {
            Object obj = ctx.lookup( name );
            home = (InventoryHome) narrow( obj, InventoryHome.class );
            System.out.println( "Retrieved home stub \"" + name + "\"." );
        }
        catch( Exception e ) {
            System.err.println( "Home lookup failed: " + e +
                                ". Verify that bean has been correctly configured and deployed." );
            System.exit(1);
        }
        return home;
    }


    static private Object narrow( Object ref, Class c ) {
        return PortableRemoteObject.narrow( ref, c );
    }
}