package examples.managed.ejb;

//required to implement the bean
import javax.ejb.*;

//required for the adapter lookup
import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import javax.resource.cci.*;
import javax.resource.ResourceException;

//required to call the Resource Adapter for CrossWorlds
import com.crossworlds.j2eeconnector.specs.CwInteractionSpec;


/**
 * A stateless EJB that demonstrates how to retrieve information from CrossWorlds
 * through the use of the Resource Adapter for CrossWorlds.
 */
public class InventoryBean implements SessionBean
{
    private SessionContext m_ctx;
    private String m_adapterJNDIName;


    /**
     * Retrieves the price of an item in CrossWorlds. This method executes
     * collaboration "Sample_RAforCrossWorlds_Item_to_Port" with business object
     * "Sample_RAforCrossWorlds_Item" and verb "Retrieve". It sets the value of
     * key attribute "ItemId" to the value of <code>itemId</code> before executing
     * the collaboration.
     *
     * @return a double representing the price of the item (as returned by the
     *         CrossWorlds collaboration)
     *
     * @throws InventoryException if an error occurs during the establishment
     *         of a connection to CrossWorlds or the execution of the
     *         collaboration
     */
    public double retrievePrice( String itemId )
        throws InventoryException
    {
        System.out.println( "method retrievePrice invoked for \"" + itemId + "\"." );

        MappedRecord outputRecord = null;
        Connection conn = null;
        try {
            System.out.println( "Establishing connection to Resource Adapter for CrossWorlds..." );
            ConnectionFactory connFactory = lookupConnectionFactory();
            conn = connFactory.getConnection();
            System.out.println( "Established connection." );

            outputRecord = executeRetrieve( connFactory, conn, itemId );
        }
        catch( ResourceException re ) {
            throw new InventoryException( "An error occurred accessing the Resource Adapter for CrossWorlds: " + re );
        }
        finally {
            //no matter what, try to close the connection so that we do not end up having memory leaks
            try { conn.close(); } catch( Exception e ) { }
            conn = null;
        }

        double price = extractPriceFromItemRecord( outputRecord );
        return price;
    }


    /**
     * Extracts and converts attribute "Price" in <code>record</code> to a
     * <code>double</code>.
     *
     * @throws InventoryException if the attribute does not exist or cannot be
     *         parsed as a <code>double</code>.
     */
    private double extractPriceFromItemRecord( MappedRecord record )
        throws InventoryException
    {
        if( !record.containsKey( "Price" ) )
            throw new InventoryException( "Returned \"Item\" does not contain attribute \"Price\". " +
                                          "Verify that you provided a value for that attribute in your collaboration:\n" + record );

        String priceAsString = record.get( "Price" ) + "";
        double price;
        try {
            price = Double.parseDouble( priceAsString );
        }
        catch( NumberFormatException nfe ) {
            throw new InventoryException( "Attribute \"Price\" in \"Item\" is not a number: " + priceAsString );
        }

        return price;
    }


    /**
     * Executes the CrossWorlds collaboration.
     *
     * @return a <code>MappedRecord</code> instance representing the business
     *         object returned by the collaboration.
     *
     * @throws InventoryException if an error is reported by the Resource Adapter
     *         for CrossWorlds
     */
    private MappedRecord executeRetrieve( ConnectionFactory connFactory, Connection conn, String itemId )
        throws InventoryException
    {
        MappedRecord outputRecord = null;
        try {
            //create a record corresponding to the business object we want to pass to the collaboration
            RecordFactory recordFactory = connFactory.getRecordFactory();
            MappedRecord inputRecord = recordFactory.createMappedRecord( "Sample_RAforCrossWorlds_Item:Retrieve" );
            inputRecord.put( "ItemId", itemId );

            //create an interaction spec. instance specifying the collaboration we want to execute
            InteractionSpec itxSpec;
            try {
                itxSpec = new CwInteractionSpec( "Sample_RAforCrossWorlds_Item_to_Port:FromObject" );
            }
            catch( Exception e ) {
                throw new InventoryException( "Failed to create CwInteractionSpec instance: " + e );
            }

            //now we finally execute our collaboration
            Interaction interaction = conn.createInteraction();
            outputRecord = (MappedRecord) interaction.execute( itxSpec, inputRecord );
        }
        catch( ResourceException re ) {
            throw new InventoryException( "The Resource Adapter for CrossWorlds reported an error" +
                                          " executing the collaboration: " + re );
        }
        return outputRecord;
    }


    /**
     * Looks up (in JNDI) a <code>ConnectionFactory</code> instance corresponding to
     * the Resource Adapter for CrossWorlds.
     *
     * @return a connection factory for the resource adapter
     *
     * @throws InventoryException if the specified entry does not exist in the
     *         JNDI context or the JNDI provider reports an error during the
     *         lookup
     */
    private ConnectionFactory lookupConnectionFactory()
        throws InventoryException
    {
        try {
            InitialContext ctx = new InitialContext();
            Object factory = ctx.lookup( m_adapterJNDIName );
            ctx.close();
            return (ConnectionFactory) narrow( factory, ConnectionFactory.class );
        }
        catch( Exception e ) {
            throw new InventoryException( "An error occurred looking up the resource adapter connection factory \"" +
                                            m_adapterJNDIName + "\": " + e + ". Verify that adapter has been deployed successfully" );
        }
    }


    static private Object narrow( Object ref, Class c ) {
        return PortableRemoteObject.narrow( ref, c );
    }


    /**
     * Retrieves the value of a property stored in the deployment descriptor of
     * this EJB.
     */
    private Object lookupProperty( String propName ) throws CreateException {
        try {
            InitialContext ic = new InitialContext();
            Object value = ic.lookup( "java:comp/env/" + propName );
            ic.close();
            return value;
        }
        catch( NamingException ne ) {
            throw new CreateException( ne + "\nVerify that property \"" + propName +
                                       "\" has been specified in the deployment descriptor of the EJB"  );
        }
    }


    /**
     * Initializes the EJB.
     */
    public void ejbCreate() throws CreateException {
        m_adapterJNDIName = (String) lookupProperty( "AdapterJNDIName" );
    }


    public void setSessionContext( SessionContext ctx ) throws EJBException {
        m_ctx = ctx;
    }


    public void ejbActivate() throws EJBException {
        //do nothing
    }


    public void ejbPassivate() throws EJBException {
        //do nothing
    }


    public void ejbRemove() throws EJBException {
        //do nothing
    }

}